@@ -4,10 +4,16 @@
 
 #include "config.h"
 
-#include <glib.h>
+G_BEGIN_DECLS
+
+void gabble_debug_set_log_file_from_env (void);
+
+G_END_DECLS
 
 #ifdef ENABLE_DEBUG
 
+#include <glib.h>
+
 G_BEGIN_DECLS
 
 typedef enum
@@ -24,6 +30,9 @@ typedef enum
   GABBLE_DEBUG_IM            = 1 << 9,
   GABBLE_DEBUG_PERSIST       = 1 << 10,
   GABBLE_DEBUG_VCARD         = 1 << 11,
+  GABBLE_DEBUG_PIPELINE      = 1 << 12,
+  GABBLE_DEBUG_JID           = 1 << 13,
+  GABBLE_DEBUG_OLPC          = 1 << 14,
 } GabbleDebugFlags;
 
 void gabble_debug_set_flags_from_env ();
@@ -32,17 +41,19 @@ gboolean gabble_debug_flag_is_set (GabbleDebugFlags flag);
 void gabble_debug (GabbleDebugFlags flag, const gchar *format, ...)
     G_GNUC_PRINTF (2, 3);
 
+G_END_DECLS
+
 #ifdef DEBUG_FLAG
 
 #define DEBUG(format, ...) \
-  gabble_debug(DEBUG_FLAG, "%s: " format, G_STRFUNC, ##__VA_ARGS__)
+  gabble_debug (DEBUG_FLAG, "%s: " format, G_STRFUNC, ##__VA_ARGS__)
 
-#define DEBUGGING gabble_debug_flag_is_set(DEBUG_FLAG)
+#define DEBUGGING gabble_debug_flag_is_set (DEBUG_FLAG)
 
 #define NODE_DEBUG(n, s) \
 G_STMT_START { \
   gchar *debug_tmp = lm_message_node_to_string (n); \
-  gabble_debug (DEBUG_FLAG, "%s: " s ":\n%s", G_STRFUNC, debug_tmp); \
+  gabble_debug (DEBUG_FLAG, "%s: %s:\n%s", G_STRFUNC, s, debug_tmp); \
   g_free (debug_tmp); \
 } G_STMT_END
 
@@ -52,17 +63,15 @@ G_STMT_START { \
 
 #ifdef DEBUG_FLAG
 
-#define DEBUG(format, ...)
+#define DEBUG(format, ...) G_STMT_START { } G_STMT_END
 
 #define DEBUGGING 0
 
-#define NODE_DEBUG(n, s)
+#define NODE_DEBUG(n, s) G_STMT_START { } G_STMT_END
 
 #endif /* DEBUG_FLAG */
 
 #endif /* ENABLE_DEBUG */
 
-G_END_DECLS
-
-#endif
+#endif /* __DEBUG_H__ */
 
