@@ -19,6 +19,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "gabble-media-session.h"
+
 #include <dbus/dbus-glib.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -27,28 +29,33 @@
 
 #define DEBUG_FLAG GABBLE_DEBUG_MEDIA
 
-#include "ansi.h"
+#include <telepathy-glib/debug-ansi.h>
 #include "debug.h"
-#include "handles.h"
 #include "namespaces.h"
 #include "util.h"
 
-#include "telepathy-errors.h"
-#include "telepathy-helpers.h"
+#include <telepathy-glib/dbus.h>
+#include <telepathy-glib/errors.h>
+#include <telepathy-glib/svc-media-interfaces.h>
 
 #include "gabble-connection.h"
 #include "gabble-media-channel.h"
 #include "gabble-media-stream.h"
-#include "gabble-presence-cache.h"
-#include "gabble-presence.h"
+#include "presence-cache.h"
+#include "presence.h"
 
-#include "gabble-media-session.h"
-#include "gabble-media-session-signals-marshal.h"
-#include "gabble-media-session-glue.h"
+#include "gabble-signals-marshal.h"
 
-G_DEFINE_TYPE(GabbleMediaSession, gabble_media_session, G_TYPE_OBJECT)
+static void session_handler_iface_init (gpointer, gpointer);
 
-#define DEFAULT_SESSION_TIMEOUT 50000
+G_DEFINE_TYPE_WITH_CODE(GabbleMediaSession,
+    gabble_media_session,
+    G_TYPE_OBJECT,
+    G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_MEDIA_SESSION_HANDLER,
+      session_handler_iface_init)
+    )
+
+#define DEFAULT_SESSION_TIMEOUT 60000
 
 #define GTALK_STREAM_NAME "gtalk"
 
@@ -59,7 +66,6 @@ G_DEFINE_TYPE(GabbleMediaSession, gabble_media_session, G_TYPE_OBJECT)
 /* signal enum */
 enum
 {
-    NEW_STREAM_HANDLER,
     STREAM_ADDED,
     TERMINATED,
     LAST_SIGNAL
@@ -95,7 +101,7 @@ struct _GabbleMediaSessionPrivate
   GPtrArray *remove_requests;
 
   gchar *id;
-  GabbleHandle peer;
+  TpHandle peer;
   gchar *peer_resource;
 
   JingleSessionState state;
@@ -118,12 +124,18 @@ typedef struct {
 
 static const SessionStateDescription session_states[] =
 {
-    { "JS_STATE_PENDING_CREATED",       ANSI_BOLD_ON ANSI_FG_BLACK ANSI_BG_WHITE   },
-    { "JS_STATE_PENDING_INITIATE_SENT", ANSI_BOLD_ON               ANSI_BG_CYAN    },
-    { "JS_STATE_PENDING_INITIATED",     ANSI_BOLD_ON               ANSI_BG_MAGENTA },
-    { "JS_STATE_PENDING_ACCEPT_SENT",   ANSI_BOLD_ON               ANSI_BG_CYAN    },
-    { "JS_STATE_ACTIVE",                ANSI_BOLD_ON               ANSI_BG_BLUE    },
-    { "JS_STATE_ENDED",                                            ANSI_BG_RED     }
+    { "JS_STATE_PENDING_CREATED",
+      TP_ANSI_BOLD_ON TP_ANSI_FG_BLACK TP_ANSI_BG_WHITE   },
+    { "JS_STATE_PENDING_INITIATE_SENT",
+      TP_ANSI_BOLD_ON                  TP_ANSI_BG_CYAN    },
+    { "JS_STATE_PENDING_INITIATED",
+      TP_ANSI_BOLD_ON                  TP_ANSI_BG_MAGENTA },
+    { "JS_STATE_PENDING_ACCEPT_SENT",
+      TP_ANSI_BOLD_ON                  TP_ANSI_BG_CYAN    },
+    { "JS_STATE_ACTIVE",
+      TP_ANSI_BOLD_ON                  TP_ANSI_BG_BLUE    },
+    { "JS_STATE_ENDED",
+                                       TP_ANSI_BG_RED     }
 };
 
 static void
@@ -162,8 +174,8 @@ _emit_new_stream (GabbleMediaSession *session,
 
   /* all of the streams are bidirectional from farsight's point of view, it's
    * just in the signalling they change */
-  g_signal_emit (session, signals[NEW_STREAM_HANDLER], 0, object_path, id,
-      media_type, TP_MEDIA_STREAM_DIRECTION_BIDIRECTIONAL);
+  tp_svc_media_session_handler_emit_new_stream_handler (session,
+      object_path, id, media_type, TP_MEDIA_STREAM_DIRECTION_BIDIRECTIONAL);
 
   g_free (object_path);
 }
@@ -181,7 +193,7 @@ _lookup_stream_by_name_and_initiator (GabbleMediaSession *session,
     {
       GabbleMediaStream *stream = g_ptr_array_index (priv->streams, i);
 
-      if (g_strdiff (stream->name, stream_name))
+      if (tp_strdiff (stream->name, stream_name))
         continue;
 
       if (stream_initiator != INITIATOR_INVALID &&
@@ -220,7 +232,7 @@ create_media_stream (GabbleMediaSession *session,
     {
       g_assert (priv->streams->len == 0);
       g_assert (media_type == TP_MEDIA_STREAM_TYPE_AUDIO);
-      g_assert (!g_strdiff (name, GTALK_STREAM_NAME));
+      g_assert (!tp_strdiff (name, GTALK_STREAM_NAME));
     }
 
   g_assert (priv->streams->len < MAX_STREAMS);
@@ -416,7 +428,8 @@ gabble_media_session_class_init (GabbleMediaSessionClass *gabble_media_session_c
   GObjectClass *object_class = G_OBJECT_CLASS (gabble_media_session_class);
   GParamSpec *param_spec;
 
-  g_type_class_add_private (gabble_media_session_class, sizeof (GabbleMediaSessionPrivate));
+  g_type_class_add_private (gabble_media_session_class,
+      sizeof (GabbleMediaSessionPrivate));
 
   object_class->constructor = gabble_media_session_constructor;
 
@@ -436,15 +449,14 @@ gabble_media_session_class_init (GabbleMediaSessionClass *gabble_media_session_c
                                     G_PARAM_STATIC_BLURB);
   g_object_class_install_property (object_class, PROP_CONNECTION, param_spec);
 
-  param_spec = g_param_spec_object ("media-channel", "GabbleMediaChannel object",
-                                    "Gabble media channel object that owns this "
-                                    "media session object.",
-                                    GABBLE_TYPE_MEDIA_CHANNEL,
-                                    G_PARAM_CONSTRUCT_ONLY |
-                                    G_PARAM_READWRITE |
-                                    G_PARAM_STATIC_NICK |
-                                    G_PARAM_STATIC_BLURB);
-  g_object_class_install_property (object_class, PROP_MEDIA_CHANNEL, param_spec);
+  param_spec = g_param_spec_object ("media-channel",
+      "GabbleMediaChannel object",
+      "Gabble media channel object that owns this media session object.",
+      GABBLE_TYPE_MEDIA_CHANNEL,
+      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_NICK |
+      G_PARAM_STATIC_BLURB);
+  g_object_class_install_property (object_class, PROP_MEDIA_CHANNEL,
+      param_spec);
 
   param_spec = g_param_spec_string ("object-path", "D-Bus object path",
                                     "The D-Bus object path used for this "
@@ -479,7 +491,7 @@ gabble_media_session_class_init (GabbleMediaSessionClass *gabble_media_session_c
   g_object_class_install_property (object_class, PROP_INITIATOR, param_spec);
 
   param_spec = g_param_spec_uint ("peer", "Session peer",
-                                  "The GabbleHandle representing the contact "
+                                  "The TpHandle representing the contact "
                                   "with whom this session communicates.",
                                   0, G_MAXUINT32, 0,
                                   G_PARAM_CONSTRUCT_ONLY |
@@ -509,15 +521,6 @@ gabble_media_session_class_init (GabbleMediaSessionClass *gabble_media_session_c
                                   G_PARAM_STATIC_BLURB);
   g_object_class_install_property (object_class, PROP_STATE, param_spec);
 
-  signals[NEW_STREAM_HANDLER] =
-    g_signal_new ("new-stream-handler",
-                  G_OBJECT_CLASS_TYPE (gabble_media_session_class),
-                  G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
-                  0,
-                  NULL, NULL,
-                  gabble_media_session_marshal_VOID__STRING_UINT_UINT_UINT,
-                  G_TYPE_NONE, 4, DBUS_TYPE_G_OBJECT_PATH, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT);
-
   signals[STREAM_ADDED] =
     g_signal_new ("stream-added",
                   G_OBJECT_CLASS_TYPE (gabble_media_session_class),
@@ -533,10 +536,8 @@ gabble_media_session_class_init (GabbleMediaSessionClass *gabble_media_session_c
                   G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                   0,
                   NULL, NULL,
-                  gabble_media_session_marshal_VOID__UINT_UINT,
+                  gabble_marshal_VOID__UINT_UINT,
                   G_TYPE_NONE, 2, G_TYPE_UINT, G_TYPE_UINT);
-
-  dbus_g_object_type_install_info (G_TYPE_FROM_CLASS (gabble_media_session_class), &dbus_glib_gabble_media_session_object_info);
 }
 
 static void
@@ -594,19 +595,14 @@ gabble_media_session_finalize (GObject *object)
  *
  * Implements D-Bus method Error
  * on interface org.freedesktop.Telepathy.Media.SessionHandler
- *
- * @error: Used to return a pointer to a GError detailing any error
- *         that occurred, D-Bus will throw the error only if this
- *         function returns FALSE.
- *
- * Returns: TRUE if successful, FALSE if an error was thrown.
  */
-gboolean
-gabble_media_session_error (GabbleMediaSession *self,
+static void
+gabble_media_session_error (TpSvcMediaSessionHandler *iface,
                             guint errno,
                             const gchar *message,
-                            GError **error)
+                            DBusGMethodInvocation *context)
 {
+  GabbleMediaSession *self = GABBLE_MEDIA_SESSION (iface);
   GabbleMediaSessionPrivate *priv;
   GPtrArray *tmp;
   guint i;
@@ -620,14 +616,16 @@ gabble_media_session_error (GabbleMediaSession *self,
 
   if (priv->state == JS_STATE_ENDED)
     {
-      return TRUE;
+      tp_svc_media_session_handler_return_from_error (context);
+      return;
     }
   else if (priv->state == JS_STATE_PENDING_CREATED)
     {
       /* shortcut to prevent sending remove actions if we haven't sent an
        * initiate yet */
       g_object_set (self, "state", JS_STATE_ENDED, NULL);
-      return TRUE;
+      tp_svc_media_session_handler_return_from_error (context);
+      return;
     }
 
   g_assert (priv->streams != NULL);
@@ -637,14 +635,14 @@ gabble_media_session_error (GabbleMediaSession *self,
 
   for (i = 0; i < tmp->len; i++)
     {
-      GabbleMediaStream *stream = g_ptr_array_index (priv->streams, i);
+      GabbleMediaStream *stream = g_ptr_array_index (tmp, i);
 
       gabble_media_stream_error (stream, errno, message, NULL);
     }
 
   g_ptr_array_free (tmp, TRUE);
 
-  return TRUE;
+  tp_svc_media_session_handler_return_from_error (context);
 }
 
 
@@ -653,17 +651,12 @@ gabble_media_session_error (GabbleMediaSession *self,
  *
  * Implements D-Bus method Ready
  * on interface org.freedesktop.Telepathy.Media.SessionHandler
- *
- * @error: Used to return a pointer to a GError detailing any error
- *         that occurred, D-Bus will throw the error only if this
- *         function returns FALSE.
- *
- * Returns: TRUE if successful, FALSE if an error was thrown.
  */
-gboolean
-gabble_media_session_ready (GabbleMediaSession *self,
-                            GError **error)
+static void
+gabble_media_session_ready (TpSvcMediaSessionHandler *iface,
+                            DBusGMethodInvocation *context)
 {
+  GabbleMediaSession *self = GABBLE_MEDIA_SESSION (iface);
   GabbleMediaSessionPrivate *priv;
   guint i;
 
@@ -676,7 +669,7 @@ gabble_media_session_ready (GabbleMediaSession *self,
   for (i = 0; i < priv->streams->len; i++)
     _emit_new_stream (self, g_ptr_array_index (priv->streams, i));
 
-  return TRUE;
+  tp_svc_media_session_handler_return_from_ready (context);
 }
 
 
@@ -718,8 +711,8 @@ _handle_create (GabbleMediaSession *session,
           else
             {
               g_set_error (error, GABBLE_XMPP_ERROR, XMPP_ERROR_CONFLICT,
-                  "session initiator is creating a stream named \"%s\" already",
-                  stream_name);
+                  "session initiator is creating a stream named \"%s\" "
+                  "already", stream_name);
               return FALSE;
             }
         }
@@ -834,21 +827,21 @@ _senders_to_direction (GabbleMediaSession *session,
 {
   TpMediaStreamDirection ret = TP_MEDIA_STREAM_DIRECTION_NONE;
 
-  if (!g_strdiff (senders, "initiator"))
+  if (!tp_strdiff (senders, "initiator"))
     {
       if (session->initiator == INITIATOR_LOCAL)
         ret = TP_MEDIA_STREAM_DIRECTION_SEND;
       else
         ret = TP_MEDIA_STREAM_DIRECTION_RECEIVE;
     }
-  else if (!g_strdiff (senders, "responder"))
+  else if (!tp_strdiff (senders, "responder"))
     {
       if (session->initiator == INITIATOR_REMOTE)
         ret = TP_MEDIA_STREAM_DIRECTION_SEND;
       else
         ret = TP_MEDIA_STREAM_DIRECTION_RECEIVE;
     }
-  else if (!g_strdiff (senders, "both"))
+  else if (!tp_strdiff (senders, "both"))
     {
       ret = TP_MEDIA_STREAM_DIRECTION_BIDIRECTIONAL;
     }
@@ -1253,14 +1246,14 @@ _call_handlers_on_stream (GabbleMediaSession *session,
 static JingleInitiator
 _creator_to_initiator (GabbleMediaSession *session, const gchar *creator)
 {
-  if (!g_strdiff (creator, "initiator"))
+  if (!tp_strdiff (creator, "initiator"))
     {
       if (session->initiator == INITIATOR_LOCAL)
         return INITIATOR_LOCAL;
       else
         return INITIATOR_REMOTE;
     }
-  else if (!g_strdiff (creator, "responder"))
+  else if (!tp_strdiff (creator, "responder"))
     {
       if (session->initiator == INITIATOR_LOCAL)
         return INITIATOR_REMOTE;
@@ -1296,7 +1289,7 @@ _call_handlers_on_streams (GabbleMediaSession *session,
       const gchar *stream_name, *stream_creator;
       JingleInitiator stream_initiator;
 
-      if (g_strdiff (content_node->name, "content"))
+      if (tp_strdiff (content_node->name, "content"))
         continue;
 
       stream_name = lm_message_node_get_attribute (content_node, "name");
@@ -1426,8 +1419,12 @@ timeout_session (gpointer data)
 
   DEBUG ("session timed out");
 
-  _gabble_media_session_terminate (session, INITIATOR_LOCAL,
-      TP_CHANNEL_GROUP_CHANGE_REASON_ERROR);
+  if (session->initiator == INITIATOR_LOCAL)
+      _gabble_media_session_terminate (session, INITIATOR_LOCAL,
+          TP_CHANNEL_GROUP_CHANGE_REASON_NO_ANSWER);
+  else
+      _gabble_media_session_terminate (session, INITIATOR_LOCAL,
+          TP_CHANNEL_GROUP_CHANGE_REASON_ERROR);
 
   return FALSE;
 }
@@ -1445,8 +1442,8 @@ _add_ready_new_streams (GabbleMediaSession *session)
       GabbleMediaStream *stream = g_ptr_array_index (priv->streams, i);
 
       GMS_DEBUG_DUMP (session, "pondering accept-time add for stream: %s, got "
-          "local codecs: %s, initiator: %s, signalling state: %d", stream->name,
-          stream->got_local_codecs ? "true" : "false",
+          "local codecs: %s, initiator: %s, signalling state: %d",
+          stream->name, stream->got_local_codecs ? "true" : "false",
           stream->initiator == INITIATOR_LOCAL ? "local" : "remote",
           stream->signalling_state);
 
@@ -1491,8 +1488,9 @@ session_state_changed (GabbleMediaSession *session,
       g_source_remove (priv->timer_id);
       priv->timer_id = 0;
 
-      /* signal any streams to the remote end which were added locally & became
-       * ready before the session was accepted, so haven't been mentioned yet */
+      /* signal any streams to the remote end which were added locally &
+       * became ready before the session was accepted, so haven't been
+       * mentioned yet */
       _add_ready_new_streams (session);
     }
 }
@@ -1510,7 +1508,8 @@ _mark_local_streams_sent (GabbleMediaSession *session)
       if (stream->initiator == INITIATOR_REMOTE)
         continue;
 
-      GMS_DEBUG_INFO (session, "marking local stream %s as signalled", stream->name);
+      GMS_DEBUG_INFO (session, "marking local stream %s as signalled",
+          stream->name);
 
       g_object_set (stream, "signalling-state", STREAM_SIG_STATE_SENT, NULL);
     }
@@ -1532,7 +1531,8 @@ _mark_local_streams_acked (GabbleMediaSession *session)
       if (stream->signalling_state != STREAM_SIG_STATE_SENT)
         continue;
 
-      GMS_DEBUG_INFO (session, "marking local stream %s as acknowledged", stream->name);
+      GMS_DEBUG_INFO (session, "marking local stream %s as acknowledged",
+          stream->name);
 
       g_object_set (stream,
           "signalling-state", STREAM_SIG_STATE_ACKNOWLEDGED,
@@ -1553,7 +1553,8 @@ _set_remote_streams_playing (GabbleMediaSession *session)
       if (stream->initiator == INITIATOR_LOCAL)
         continue;
 
-      GMS_DEBUG_INFO (session, "setting remote stream %s as playing", stream->name);
+      GMS_DEBUG_INFO (session, "setting remote stream %s as playing",
+          stream->name);
 
       g_object_set (stream, "playing", TRUE, NULL);
     }
@@ -2084,19 +2085,23 @@ stream_got_local_codecs_changed_cb (GabbleMediaStream *stream,
 
 static gchar *
 get_jid_for_contact (GabbleMediaSession *session,
-                     GabbleHandle handle)
+                     TpHandle handle)
 {
   GabbleMediaSessionPrivate *priv;
+  TpBaseConnection *conn;
   const gchar *base_jid;
-  GabbleHandle self;
+  TpHandle self;
+  TpHandleRepoIface *contact_handles;
 
   g_assert (GABBLE_IS_MEDIA_SESSION (session));
 
   priv = GABBLE_MEDIA_SESSION_GET_PRIVATE (session);
-  self = priv->conn->self_handle;
+  conn = (TpBaseConnection *)priv->conn;
+  contact_handles = tp_base_connection_get_handles (conn,
+      TP_HANDLE_TYPE_CONTACT);
+  self = conn->self_handle;
 
-  base_jid = gabble_handle_inspect (priv->conn->handles,
-      TP_HANDLE_TYPE_CONTACT, handle);
+  base_jid = tp_handle_inspect (contact_handles, handle);
   g_assert (base_jid != NULL);
 
   if (handle == self)
@@ -2121,15 +2126,17 @@ _gabble_media_session_message_new (GabbleMediaSession *session,
                                    LmMessageNode **session_node)
 {
   GabbleMediaSessionPrivate *priv;
+  TpBaseConnection *conn;
   LmMessage *msg;
   LmMessageNode *iq_node, *node;
   gchar *peer_jid, *initiator_jid;
-  GabbleHandle initiator_handle;
+  TpHandle initiator_handle;
   const gchar *element, *xmlns;
 
   g_assert (GABBLE_IS_MEDIA_SESSION (session));
 
   priv = GABBLE_MEDIA_SESSION_GET_PRIVATE (session);
+  conn = (TpBaseConnection *)priv->conn;
 
   peer_jid = get_jid_for_contact (session, priv->peer);
 
@@ -2148,7 +2155,7 @@ _gabble_media_session_message_new (GabbleMediaSession *session,
     element = "jingle";
 
   if (session->initiator == INITIATOR_LOCAL)
-    initiator_handle = priv->conn->self_handle;
+    initiator_handle = conn->self_handle;
   else
     initiator_handle = priv->peer;
 
@@ -2222,7 +2229,8 @@ content_remove_msg_reply_cb (GabbleConnection *conn,
   GPtrArray *removing = (GPtrArray *) user_data;
   guint i;
 
-  MSG_REPLY_CB_END_SESSION_IF_NOT_SUCCESSFUL (session, "stream removal failed");
+  MSG_REPLY_CB_END_SESSION_IF_NOT_SUCCESSFUL (session,
+      "stream removal failed");
 
   for (i = 0; i < removing->len; i++)
     destroy_media_stream (session,
@@ -2401,7 +2409,8 @@ _gabble_media_session_terminate (GabbleMediaSession *session,
                                  TpChannelGroupChangeReason why)
 {
   GabbleMediaSessionPrivate *priv = GABBLE_MEDIA_SESSION_GET_PRIVATE (session);
-  GabbleHandle actor;
+  TpBaseConnection *conn = (TpBaseConnection *)priv->conn;
+  TpHandle actor;
 
   if (priv->state == JS_STATE_ENDED)
     return;
@@ -2412,7 +2421,7 @@ _gabble_media_session_terminate (GabbleMediaSession *session,
     }
   else
     {
-      actor = priv->conn->self_handle;
+      actor = conn->self_handle;
 
       /* Need to tell them that it's all over. */
 
@@ -2475,19 +2484,19 @@ _gabble_media_session_debug (GabbleMediaSession *session,
 
       switch (type) {
         case DEBUG_MSG_INFO:
-          type_str = ANSI_BOLD_ON ANSI_FG_WHITE;
+          type_str = TP_ANSI_BOLD_ON TP_ANSI_FG_WHITE;
           break;
         case DEBUG_MSG_DUMP:
-          type_str = ANSI_BOLD_ON ANSI_FG_GREEN;
+          type_str = TP_ANSI_BOLD_ON TP_ANSI_FG_GREEN;
           break;
         case DEBUG_MSG_WARNING:
-          type_str = ANSI_BOLD_ON ANSI_FG_YELLOW;
+          type_str = TP_ANSI_BOLD_ON TP_ANSI_FG_YELLOW;
           break;
         case DEBUG_MSG_ERROR:
-          type_str = ANSI_BOLD_ON ANSI_FG_WHITE ANSI_BG_RED;
+          type_str = TP_ANSI_BOLD_ON TP_ANSI_FG_WHITE TP_ANSI_BG_RED;
           break;
         case DEBUG_MSG_EVENT:
-          type_str = ANSI_BOLD_ON ANSI_FG_CYAN;
+          type_str = TP_ANSI_BOLD_ON TP_ANSI_FG_CYAN;
           break;
         default:
           g_assert_not_reached ();
@@ -2495,15 +2504,15 @@ _gabble_media_session_debug (GabbleMediaSession *session,
       }
 
       printf ("[%s%s%s] %s%-26s%s %s%s%s\n",
-          ANSI_BOLD_ON ANSI_FG_WHITE,
+          TP_ANSI_BOLD_ON TP_ANSI_FG_WHITE,
           stamp,
-          ANSI_RESET,
+          TP_ANSI_RESET,
           session_states[priv->state].attributes,
           session_states[priv->state].name,
-          ANSI_RESET,
+          TP_ANSI_RESET,
           type_str,
           buf,
-          ANSI_RESET);
+          TP_ANSI_RESET);
 
       fflush (stdout);
     }
@@ -2573,7 +2582,7 @@ _gabble_media_session_request_streams (GabbleMediaSession *session,
 
   if (presence == NULL)
     {
-      g_set_error (error, TELEPATHY_ERRORS, NotAvailable,
+      g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
           "member has no audio/video capabilities");
 
       return FALSE;
@@ -2599,7 +2608,7 @@ _gabble_media_session_request_streams (GabbleMediaSession *session,
         }
       else
         {
-          g_set_error (error, TELEPATHY_ERRORS, InvalidArgument,
+          g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
               "given media type %u is invalid", media_type);
           return FALSE;
         }
@@ -2628,7 +2637,7 @@ _gabble_media_session_request_streams (GabbleMediaSession *session,
 
           g_assert (priv->streams->len == 1);
 
-          g_set_error (error, TELEPATHY_ERRORS, NotAvailable,
+          g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
               "Google Talk calls may only contain one stream");
 
           return FALSE;
@@ -2640,7 +2649,7 @@ _gabble_media_session_request_streams (GabbleMediaSession *session,
           GMS_DEBUG_INFO (session,
             "in Jingle mode but have insufficient caps for requested streams");
 
-          g_set_error (error, TELEPATHY_ERRORS, NotAvailable,
+          g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
               "existing call member doesn't support all requested media"
               " types");
 
@@ -2693,7 +2702,7 @@ _gabble_media_session_request_streams (GabbleMediaSession *session,
                     }
                   else
                     {
-                      g_set_error (error, TELEPATHY_ERRORS, NotAvailable,
+                      g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
                           "Google Talk calls may only contain one stream");
 
                       return FALSE;
@@ -2712,7 +2721,7 @@ _gabble_media_session_request_streams (GabbleMediaSession *session,
           GMS_DEBUG_INFO (session,
             "contact doesn't have a resource with suitable capabilities");
 
-          g_set_error (error, TELEPATHY_ERRORS, NotAvailable,
+          g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
               "member does not have the desired audio/video capabilities");
 
           return FALSE;
@@ -2724,7 +2733,7 @@ _gabble_media_session_request_streams (GabbleMediaSession *session,
   /* check it's not a ridiculous number of streams */
   if ((priv->streams->len + media_types->len) > MAX_STREAMS)
     {
-      g_set_error (error, TELEPATHY_ERRORS, NotAvailable,
+      g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
           "I think that's quite enough streams already");
       return FALSE;
     }
@@ -2796,7 +2805,8 @@ direction_msg_reply_cb (GabbleConnection *conn,
   GabbleMediaSession *session = GABBLE_MEDIA_SESSION (user_data);
   GabbleMediaStream *stream = GABBLE_MEDIA_STREAM (object);
 
-  MSG_REPLY_CB_END_SESSION_IF_NOT_SUCCESSFUL (session, "direction change failed");
+  MSG_REPLY_CB_END_SESSION_IF_NOT_SUCCESSFUL (session,
+      "direction change failed");
 
   if (stream->playing)
     {
@@ -2871,7 +2881,7 @@ _gabble_media_session_request_stream_direction (GabbleMediaSession *session,
       if (requested_dir == TP_MEDIA_STREAM_DIRECTION_BIDIRECTIONAL)
         return TRUE;
 
-      g_set_error (error, TELEPATHY_ERRORS, NotAvailable,
+      g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
           "Google Talk calls can only be bi-directional");
       return FALSE;
     }
@@ -2924,3 +2934,15 @@ _gabble_media_session_request_stream_direction (GabbleMediaSession *session,
   return send_direction_change (session, stream, requested_dir, error);
 }
 
+static void
+session_handler_iface_init (gpointer g_iface, gpointer iface_data)
+{
+  TpSvcMediaSessionHandlerClass *klass =
+    (TpSvcMediaSessionHandlerClass *)g_iface;
+
+#define IMPLEMENT(x) tp_svc_media_session_handler_implement_##x (\
+    klass, gabble_media_session_##x)
+  IMPLEMENT(error);
+  IMPLEMENT(ready);
+#undef IMPLEMENT
+}
