@@ -17,6 +17,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "muc-factory.h"
+
 #define DBUS_API_SUBJECT_TO_CHANGE
 
 #include <stdlib.h>
@@ -35,21 +37,21 @@
 #include "debug.h"
 #include "disco.h"
 #include "gabble-connection.h"
-#include "gabble-presence-cache.h"
+#include "presence-cache.h"
 #include "gabble-muc-channel.h"
 #include "gabble-roomlist-channel.h"
-#include "handles.h"
-#include "muc-factory.h"
 #include "namespaces.h"
-#include "telepathy-interfaces.h"
+#include <telepathy-glib/interfaces.h>
 #include "text-mixin.h"
-#include "tp-channel-factory-iface.h"
+#include <telepathy-glib/channel-factory-iface.h>
 #include "util.h"
 
-static void gabble_muc_factory_iface_init (gpointer g_iface, gpointer iface_data);
+static void gabble_muc_factory_iface_init (gpointer g_iface,
+    gpointer iface_data);
 
 G_DEFINE_TYPE_WITH_CODE (GabbleMucFactory, gabble_muc_factory, G_TYPE_OBJECT,
-    G_IMPLEMENT_INTERFACE (TP_TYPE_CHANNEL_FACTORY_IFACE, gabble_muc_factory_iface_init));
+    G_IMPLEMENT_INTERFACE (TP_TYPE_CHANNEL_FACTORY_IFACE,
+      gabble_muc_factory_iface_init));
 
 /* properties */
 enum
@@ -74,9 +76,12 @@ struct _GabbleMucFactoryPrivate
   gboolean dispose_has_run;
 };
 
-#define GABBLE_MUC_FACTORY_GET_PRIVATE(o)    (G_TYPE_INSTANCE_GET_PRIVATE ((o), GABBLE_TYPE_MUC_FACTORY, GabbleMucFactoryPrivate))
+#define GABBLE_MUC_FACTORY_GET_PRIVATE(o) \
+  (G_TYPE_INSTANCE_GET_PRIVATE ((o), GABBLE_TYPE_MUC_FACTORY, \
+                                GabbleMucFactoryPrivate))
 
-static GObject *gabble_muc_factory_constructor (GType type, guint n_props, GObjectConstructParam *props);
+static GObject *gabble_muc_factory_constructor (GType type, guint n_props,
+    GObjectConstructParam *props);
 
 static void
 gabble_muc_factory_init (GabbleMucFactory *fac)
@@ -133,7 +138,8 @@ gabble_muc_factory_dispose (GObject *object)
 
   tp_channel_factory_iface_close_all (TP_CHANNEL_FACTORY_IFACE (object));
 
-  g_hash_table_foreach (priv->disco_requests, cancel_disco_request, priv->conn->disco);
+  g_hash_table_foreach (priv->disco_requests, cancel_disco_request,
+      priv->conn->disco);
   g_hash_table_destroy (priv->disco_requests);
 
   if (G_OBJECT_CLASS (gabble_muc_factory_parent_class)->dispose)
@@ -184,7 +190,8 @@ gabble_muc_factory_class_init (GabbleMucFactoryClass *gabble_muc_factory_class)
   GObjectClass *object_class = G_OBJECT_CLASS (gabble_muc_factory_class);
   GParamSpec *param_spec;
 
-  g_type_class_add_private (gabble_muc_factory_class, sizeof (GabbleMucFactoryPrivate));
+  g_type_class_add_private (gabble_muc_factory_class,
+      sizeof (GabbleMucFactoryPrivate));
 
   object_class->constructor = gabble_muc_factory_constructor;
   object_class->dispose = gabble_muc_factory_dispose;
@@ -208,15 +215,21 @@ static GabbleMucChannel *
 get_muc_from_jid (GabbleMucFactory *fac, const gchar *jid)
 {
   GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (fac);
-  GabbleHandle handle;
+  TpBaseConnection *conn = (TpBaseConnection *)priv->conn;
+  TpHandleRepoIface *room_repo = tp_base_connection_get_handles (conn,
+      TP_HANDLE_TYPE_ROOM);
+  TpHandle handle;
   GabbleMucChannel *chan = NULL;
+  gchar *room;
 
-  if (gabble_handle_for_room_exists (priv->conn->handles, jid, TRUE))
-    {
-      handle = gabble_handle_for_room (priv->conn->handles, jid);
+  room = gabble_remove_resource (jid);
+  if (!room)
+    return NULL;
 
-      chan = g_hash_table_lookup (priv->channels, GUINT_TO_POINTER (handle));
-    }
+  handle = tp_handle_lookup (room_repo, room, NULL, NULL);
+  g_free (room);
+  if (handle)
+    chan = g_hash_table_lookup (priv->channels, GUINT_TO_POINTER (handle));
 
   return chan;
 }
@@ -233,7 +246,7 @@ muc_channel_closed_cb (GabbleMucChannel *chan, gpointer user_data)
 {
   GabbleMucFactory *fac = GABBLE_MUC_FACTORY (user_data);
   GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (fac);
-  GabbleHandle room_handle;
+  TpHandle room_handle;
 
   if (priv->channels != NULL)
     {
@@ -253,7 +266,8 @@ muc_ready_cb (GabbleMucChannel *chan,
 
   DEBUG ("chan=%p", chan);
 
-  g_signal_emit_by_name (fac, "new-channel", chan);
+  tp_channel_factory_iface_emit_new_channel (fac, (TpChannelIface *)chan,
+      NULL);
 }
 
 static void
@@ -265,22 +279,25 @@ muc_join_error_cb (GabbleMucChannel *chan,
 
   DEBUG ("error->code=%u, error->message=\"%s\"", error->code, error->message);
 
-  g_signal_emit_by_name (fac, "channel-error", chan, error);
+  tp_channel_factory_iface_emit_channel_error (fac, (TpChannelIface *)chan,
+      error, NULL);
 }
 
 /**
  * new_muc_channel
  */
 static GabbleMucChannel *
-new_muc_channel (GabbleMucFactory *fac, GabbleHandle handle, gboolean invite_self)
+new_muc_channel (GabbleMucFactory *fac, TpHandle handle, gboolean invite_self)
 {
   GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (fac);
+  TpBaseConnection *conn = (TpBaseConnection *)priv->conn;
   GabbleMucChannel *chan;
   char *object_path;
 
   g_assert (g_hash_table_lookup (priv->channels, GINT_TO_POINTER (handle)) == NULL);
 
-  object_path = g_strdup_printf ("%s/MucChannel%u", priv->conn->object_path, handle);
+  object_path = g_strdup_printf ("%s/MucChannel%u",
+      conn->object_path, handle);
 
   DEBUG ("creating new chan, object path %s", object_path);
 
@@ -304,11 +321,42 @@ new_muc_channel (GabbleMucFactory *fac, GabbleHandle handle, gboolean invite_sel
   return chan;
 }
 
+static void
+do_invite (GabbleMucFactory *fac,
+           const gchar *room,
+           TpHandle inviter_handle,
+           const gchar *reason)
+{
+  GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (fac);
+  TpHandleRepoIface *room_repo = tp_base_connection_get_handles (
+      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_ROOM);
+  TpHandle room_handle;
+
+  room_handle = tp_handle_ensure (room_repo, room, NULL, NULL);
+
+  if (room_handle == 0)
+    {
+      DEBUG ("got a MUC invitation message with invalid room JID \"%s\"; "
+          "ignoring", room);
+      return;
+    }
+
+  if (g_hash_table_lookup (priv->channels, GUINT_TO_POINTER (room_handle)) ==
+      NULL)
+    {
+      GabbleMucChannel *chan = new_muc_channel (fac, room_handle, FALSE);
+      _gabble_muc_channel_handle_invited (chan, inviter_handle, reason);
+    }
+  else
+    {
+      DEBUG ("ignoring invite to room \"%s\"; we're already there", room);
+    }
+}
 
 struct DiscoInviteData {
     GabbleMucFactory *factory;
     gchar *reason;
-    GabbleHandle inviter;
+    TpHandle inviter;
 };
 
 /**
@@ -330,49 +378,200 @@ obsolete_invite_disco_cb (GabbleDisco *self,
 
   GabbleMucFactory *fac = GABBLE_MUC_FACTORY (data->factory);
   GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (fac);
+  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
+      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
   LmMessageNode *identity;
-  const char *category, *type;
-  GabbleHandle handle;
+  const char *category = NULL, *type = NULL;
 
   g_hash_table_remove (priv->disco_requests, request);
 
-  identity = lm_message_node_get_child (query_result, "identity");
-  if (NULL == identity)
-    return;
-
-  category = lm_message_node_get_attribute (identity, "category");
-  if (NULL == category)
-    return;
-
-  type = lm_message_node_get_attribute (identity, "type");
-  if (NULL == type)
-    return;
-
-  if (0 != strcmp (category, "conference") ||
-      0 != strcmp (type, "text"))
+  if (error != NULL)
     {
-      DEBUG ("obsolete invite request specified invalid jid '%s', ignoring", jid);
+      DEBUG ("ignoring obsolete invite to room '%s'; got disco error: %s",
+          jid, error->message);
+      goto out;
+    }
+
+  identity = lm_message_node_get_child (query_result, "identity");
+  if (identity != NULL)
+    {
+      category = lm_message_node_get_attribute (identity, "category");
+      type = lm_message_node_get_attribute (identity, "type");
+    }
+
+  if (tp_strdiff (category, "conference") ||
+      tp_strdiff (type, "text"))
+    {
+      DEBUG ("obsolete invite request specified inappropriate jid '%s' "
+          "(not a text conference); ignoring request", jid);
+      goto out;
     }
 
   /* OK, it's MUC after all, create a new channel */
-  handle = gabble_handle_for_room (priv->conn->handles, jid);
+  do_invite (fac, jid, data->inviter, data->reason);
 
-  if (g_hash_table_lookup (priv->channels, GINT_TO_POINTER (handle)) == NULL)
+out:
+  tp_handle_unref (contact_repo, data->inviter);
+  g_free (data->reason);
+  g_slice_free (struct DiscoInviteData, data);
+}
+
+static gboolean
+process_muc_invite (GabbleMucFactory *fac,
+                    LmMessage *message,
+                    const gchar *from,
+                    TpChannelTextSendError send_error)
+{
+  GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (fac);
+  TpBaseConnection *conn = (TpBaseConnection *)priv->conn;
+  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (conn,
+      TP_HANDLE_TYPE_CONTACT);
+
+  LmMessageNode *x_node, *invite_node, *reason_node;
+  const gchar *invite_from, *reason = NULL;
+  TpHandle inviter_handle;
+  gchar *room;
+
+  /* does it have a muc subnode? */
+  x_node = lm_message_node_get_child_with_namespace (message->node, "x",
+      NS_MUC_USER);
+
+  if (x_node == NULL)
+    return FALSE;
+
+  /* and an invitation? */
+  invite_node = lm_message_node_get_child (x_node, "invite");
+
+  if (invite_node == NULL)
+    return FALSE;
+
+  /* FIXME: do something with these? */
+  if (send_error != GABBLE_TEXT_CHANNEL_SEND_NO_ERROR)
     {
-      GabbleMucChannel *chan;
-      chan = new_muc_channel (fac, handle, FALSE);
-      _gabble_muc_channel_handle_invited (chan, data->inviter, data->reason);
+      NODE_DEBUG (message->node, "got a MUC invitation message with a send "
+          "error; ignoring");
+
+      return TRUE;
+    }
+
+  invite_from = lm_message_node_get_attribute (invite_node, "from");
+  if (invite_from == NULL)
+    {
+      NODE_DEBUG (message->node, "got a MUC invitation message with no JID; "
+          "ignoring");
+
+      return TRUE;
+    }
+
+  inviter_handle = tp_handle_ensure (contact_repo, invite_from,
+      NULL, NULL);
+  if (inviter_handle == 0)
+    {
+      NODE_DEBUG (message->node, "got a MUC invitation message with invalid "
+          "inviter JID; ignoring");
+
+      return TRUE;
+    }
+
+  reason_node = lm_message_node_get_child (invite_node, "reason");
+
+  if (reason_node != NULL)
+    reason = lm_message_node_get_value (reason_node);
+
+  if (reason == NULL)
+    reason = "";
+
+  /* create the channel */
+  room = gabble_remove_resource (from);
+  do_invite (fac, room, inviter_handle, reason);
+  g_free (room);
+
+  tp_handle_unref (contact_repo, inviter_handle);
+
+  return TRUE;
+}
+
+static gboolean
+process_obsolete_invite (GabbleMucFactory *fac,
+                         LmMessage *message,
+                         const gchar *from,
+                         const gchar *body,
+                         TpChannelTextSendError send_error)
+{
+  GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (fac);
+  TpBaseConnection *conn = (TpBaseConnection *)priv->conn;
+  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (conn,
+      TP_HANDLE_TYPE_CONTACT);
+
+  LmMessageNode *x_node;
+  const gchar *room;
+  TpHandle inviter_handle;
+  GabbleDiscoRequest *request;
+  struct DiscoInviteData *disco_udata;
+
+  /* check for obsolete invite method */
+  x_node = lm_message_node_get_child_with_namespace (message->node, "x",
+      NS_X_CONFERENCE);
+  if (x_node == NULL)
+    return FALSE;
+
+  /* this can only happen if the user sent an obsolete invite with another
+   * client or something */
+  if (send_error != GABBLE_TEXT_CHANNEL_SEND_NO_ERROR)
+    {
+      NODE_DEBUG (message->node, "got an obsolete MUC invitation message with "
+          "a send error; ignoring");
+
+      return TRUE;
+    }
+
+  /* the room JID is in x */
+  room = lm_message_node_get_attribute (x_node, "jid");
+  if (room == NULL)
+    {
+      NODE_DEBUG (message->node, "got a obsolete MUC invitation with no room "
+          "JID; ignoring");
+
+      return TRUE;
+    }
+
+  /* the inviter JID is in "from" */
+  inviter_handle = tp_handle_ensure (contact_repo, from, NULL, NULL);
+  if (inviter_handle == 0)
+    {
+      NODE_DEBUG (message->node, "got an obsolete MUC invitation message from "
+          "an invalid JID; ignoring");
+
+      return TRUE;
+    }
+
+  disco_udata = g_slice_new0 (struct DiscoInviteData);
+  disco_udata->factory = fac;
+  disco_udata->reason = g_strdup (body);
+  disco_udata->inviter = inviter_handle;
+
+  DEBUG ("received obsolete MUC invite from handle %u (%s), discoing room %s",
+      inviter_handle, from, room);
+
+  request = gabble_disco_request (priv->conn->disco, GABBLE_DISCO_TYPE_INFO,
+      room, NULL, obsolete_invite_disco_cb, disco_udata, G_OBJECT (fac), NULL);
+
+  if (request != NULL)
+    {
+      g_hash_table_insert (priv->disco_requests, request, NULL);
     }
   else
     {
-      DEBUG ("ignoring invite to a room '%s' we're already in", jid);
+      DEBUG ("obsolete MUC invite disco failed, freeing info");
+
+      tp_handle_unref (contact_repo, inviter_handle);
+      g_free (disco_udata->reason);
+      g_slice_free (struct DiscoInviteData, disco_udata);
     }
 
-  g_free (data->reason);
-  g_free (data);
+  return TRUE;
 }
 
-
 /**
  * muc_factory_message_cb:
  *
@@ -387,171 +586,114 @@ muc_factory_message_cb (LmMessageHandler *handler,
 {
   GabbleMucFactory *fac = GABBLE_MUC_FACTORY (user_data);
   GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (fac);
+  TpBaseConnection *conn = (TpBaseConnection *)priv->conn;
+  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (conn,
+      TP_HANDLE_TYPE_CONTACT);
+  TpHandleRepoIface *room_repo = tp_base_connection_get_handles (conn,
+      TP_HANDLE_TYPE_ROOM);
 
-  const gchar *from, *body, *body_offset;
+  const gchar *from, *body, *subject;
   time_t stamp;
   TpChannelTextMessageType msgtype;
-  LmMessageNode *node;
+  TpHandleRepoIface *handle_source;
   TpHandleType handle_type;
-  GabbleHandle room_handle, handle;
+  TpHandle room_handle, handle;
   GabbleMucChannel *chan;
-  GabbleTextMixinSendError send_error;
+  gint state;
+  TpChannelTextSendError send_error;
+  gchar *room;
+  LmMessageNode *subj_node;
 
   if (!gabble_text_mixin_parse_incoming_message (message, &from, &stamp,
-        &msgtype, &body, &body_offset, &send_error))
-    return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
+        &msgtype, &body, &state, &send_error))
+    return LM_HANDLER_RESULT_REMOVE_MESSAGE;
 
-  /* does it have a muc subnode? */
-  node = lm_message_node_get_child_with_namespace (message->node, "x",
-      NS_MUC_USER);
-  if (node != NULL)
-    {
-      /* and an invitation? */
-      node = lm_message_node_get_child (node, "invite");
-      if (node != NULL)
-        {
-          LmMessageNode *reason_node;
-          const gchar *invite_from, *reason;
-          GabbleHandle inviter_handle;
+  if (process_muc_invite (fac, message, from, send_error))
+    return LM_HANDLER_RESULT_REMOVE_MESSAGE;
 
-          if (send_error != CHANNEL_TEXT_SEND_NO_ERROR)
-            {
-              NODE_DEBUG (message->node, "got a MUC invitation message "
-                             "with a send error; ignoring");
-
-              return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
-            }
-
-          invite_from = lm_message_node_get_attribute (node, "from");
-          if (invite_from == NULL)
-            {
-              NODE_DEBUG (message->node, "got a MUC invitation message "
-                             "without a from field on the invite node, "
-                             "ignoring");
-
-              return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
-            }
-
-          inviter_handle = gabble_handle_for_contact (priv->conn->handles,
-                                                      invite_from, FALSE);
-
-          reason_node = lm_message_node_get_child (node, "reason");
-          if (reason_node != NULL)
-            {
-              reason = lm_message_node_get_value (reason_node);
-            }
-          else
-            {
-              reason = "";
-              NODE_DEBUG (message->node, "no MUC invite reason specified");
-            }
-
-          /* create the channel */
-          handle = gabble_handle_for_room (priv->conn->handles, from);
-
-          if (g_hash_table_lookup (priv->channels, GINT_TO_POINTER (handle)) == NULL)
-            {
-              chan = new_muc_channel (fac, handle, FALSE);
-              _gabble_muc_channel_handle_invited (chan, inviter_handle, reason);
-            }
-          else
-            {
-              NODE_DEBUG (message->node, "ignoring invite to a room we're already in");
-            }
-
-          return LM_HANDLER_RESULT_REMOVE_MESSAGE;
-        }
-    }
-  else
-    {
-      GabbleHandle inviter_handle;
-      GabbleDiscoRequest *request;
-      const gchar *reason;
-      struct DiscoInviteData *disco_udata;
-
-      /* check for obsolete invite method */
-      for (node = message->node->children; node != NULL; node = node->next)
-        if (strcmp (node->name, "x") == 0)
-          if (lm_message_node_has_namespace (node, NS_X_CONFERENCE, NULL))
-            break;
-
-      if (node == NULL)
-        goto HANDLE_MESSAGE;
-
-      /* the room JID is in x */
-      from = lm_message_node_get_attribute (node, "jid");
-      if (from == NULL)
-        return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
-
-      /* the inviter JID is in "from" */
-      inviter_handle = gabble_handle_for_contact (priv->conn->handles,
-                                                  from, FALSE);
-
-      /* reason is the body */
-      reason = body;
-
-      disco_udata = g_new0 (struct DiscoInviteData, 1);
-      disco_udata->factory = fac;
-      disco_udata->reason = g_strdup (reason);
-      disco_udata->inviter = inviter_handle;
-
-      NODE_DEBUG (message->node, "received obsolete invite method");
-
-      request = gabble_disco_request (priv->conn->disco, GABBLE_DISCO_TYPE_INFO,
-          from, NULL, obsolete_invite_disco_cb, disco_udata, G_OBJECT (fac), NULL);
-
-      if (request != NULL)
-        g_hash_table_insert (priv->disco_requests, request, NULL);
-
-      return LM_HANDLER_RESULT_REMOVE_MESSAGE;
-  }
-
-HANDLE_MESSAGE:
+  if (process_obsolete_invite (fac, message, from, body, send_error))
+    return LM_HANDLER_RESULT_REMOVE_MESSAGE;
 
   /* check if a room with the jid exists */
-  if (!gabble_handle_for_room_exists (priv->conn->handles, from, TRUE))
+  room = gabble_remove_resource (from);
+  room_handle = tp_handle_lookup (room_repo, room, NULL, NULL);
+  g_free (room);
+
+  /* the message is nothing to do with MUC, do nothing */
+  if (room_handle == 0)
     {
       return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
     }
 
-  room_handle = gabble_handle_for_room (priv->conn->handles, from);
-
   /* find the MUC channel */
   chan = g_hash_table_lookup (priv->channels, GUINT_TO_POINTER (room_handle));
 
   if (chan == NULL)
     {
-      g_warning ("%s: ignoring groupchat message from known handle with "
-                 "no MUC channel", G_STRFUNC);
+      NODE_DEBUG (message->node, "ignoring MUC message from known "
+          "handle with no corresponding channel");
 
-      return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
+      return LM_HANDLER_RESULT_REMOVE_MESSAGE;
     }
 
   /* get the handle of the sender, which is either the room
    * itself or one of its members */
-  if (gabble_handle_for_room_exists (priv->conn->handles, from, FALSE))
+  if (strchr (from, '/') == NULL)
     {
+      handle_source = room_repo;
       handle_type = TP_HANDLE_TYPE_ROOM;
       handle = room_handle;
+      tp_handle_ref (room_repo, handle);
     }
   else
     {
+      handle_source = contact_repo;
       handle_type = TP_HANDLE_TYPE_CONTACT;
-      handle = gabble_handle_for_contact (priv->conn->handles, from, TRUE);
+      handle = tp_handle_ensure (contact_repo, from,
+          GUINT_TO_POINTER (GABBLE_JID_ROOM_MEMBER), NULL);
+
+      if (handle == 0)
+        {
+          NODE_DEBUG (message->node, "MUC message from invalid JID; ignoring");
+          return LM_HANDLER_RESULT_REMOVE_MESSAGE;
+        }
+
+      /* anything other than a type="groupchat" is from the person directly and
+       * simply relayed by the MUC, so should be left to the normal handlers */
+      if (lm_message_get_sub_type (message) != LM_MESSAGE_SUB_TYPE_GROUPCHAT)
+        {
+          tp_handle_unref (contact_repo, handle);
+
+          return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
+        }
     }
 
-  if (send_error != CHANNEL_TEXT_SEND_NO_ERROR)
+  if (send_error != GABBLE_TEXT_CHANNEL_SEND_NO_ERROR)
     {
-      _gabble_text_mixin_send_error_signal (G_OBJECT (chan), send_error, stamp,
-          msgtype, body_offset);
-      return LM_HANDLER_RESULT_REMOVE_MESSAGE;
+      tp_svc_channel_type_text_emit_send_error ((TpSvcChannelTypeText *) chan,
+          send_error, stamp, msgtype, body);
+      goto done;
     }
 
-  if (_gabble_muc_channel_receive (chan, msgtype, handle_type, handle, stamp,
-                                   body_offset, message))
-    return LM_HANDLER_RESULT_REMOVE_MESSAGE;
+  if (state != -1 && handle_type == TP_HANDLE_TYPE_CONTACT)
+    _gabble_muc_channel_state_receive (chan, state, handle);
 
-  return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
+  if (body != NULL)
+    _gabble_muc_channel_receive (chan, msgtype, handle_type, handle, stamp,
+        body, message);
+
+  subj_node = lm_message_node_get_child (message->node, "subject");
+  if (subj_node != NULL)
+    {
+      subject = lm_message_node_get_value (subj_node);
+      _gabble_muc_channel_handle_subject (chan, msgtype, handle_type, handle,
+          stamp, subject, message);
+    }
+
+done:
+  tp_handle_unref (handle_source, handle);
+
+  return LM_HANDLER_RESULT_REMOVE_MESSAGE;
 }
 
 
@@ -572,6 +714,8 @@ muc_factory_presence_cb (LmMessageHandler *handler,
 {
   GabbleMucFactory *fac = GABBLE_MUC_FACTORY (user_data);
   GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (fac);
+  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
+      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
   const char *from;
   LmMessageSubType sub_type;
   GabbleMucChannel *muc_chan;
@@ -583,7 +727,8 @@ muc_factory_presence_cb (LmMessageHandler *handler,
 
   if (from == NULL)
     {
-      NODE_DEBUG (msg->node, "presence stanza without from attribute, ignoring");
+      NODE_DEBUG (msg->node,
+          "presence stanza without from attribute, ignoring");
       return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
     }
 
@@ -600,24 +745,28 @@ muc_factory_presence_cb (LmMessageHandler *handler,
       return LM_HANDLER_RESULT_REMOVE_MESSAGE;
     }
 
-  x_node = lm_message_node_get_child_with_namespace (msg->node, "x", NS_MUC_USER);
+  x_node = lm_message_node_get_child_with_namespace (msg->node, "x",
+      NS_MUC_USER);
 
   /* is it a MUC member presence? */
   if (x_node != NULL)
     {
       if (muc_chan != NULL)
         {
-          GabbleHandle handle;
+          TpHandle handle;
 
-          handle = gabble_handle_for_contact (priv->conn->handles, from, TRUE);
+          handle = tp_handle_ensure (contact_repo, from,
+              GUINT_TO_POINTER (GABBLE_JID_ROOM_MEMBER), NULL);
           if (handle == 0)
             {
-              NODE_DEBUG (msg->node, "discarding MUC presence from malformed jid");
+              NODE_DEBUG (msg->node,
+                  "discarding MUC presence from malformed jid");
               return LM_HANDLER_RESULT_REMOVE_MESSAGE;
             }
 
           _gabble_muc_channel_member_presence_updated (muc_chan, handle,
                                                        msg, x_node);
+          tp_handle_unref (contact_repo, handle);
         }
       else
         {
@@ -645,32 +794,34 @@ roomlist_channel_closed_cb (GabbleRoomlistChannel *chan, gpointer data)
 }
 
 static gboolean
-make_roomlist_channel (GabbleMucFactory *fac)
+make_roomlist_channel (GabbleMucFactory *fac, gpointer request)
 {
   GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (fac);
+  TpBaseConnection *conn = (TpBaseConnection *)priv->conn;
 
-  if (priv->roomlist_channel == NULL)
-    {
-      const gchar *server;
-      gchar *object_path;
+  const gchar *server;
+  gchar *object_path;
 
-      server = _gabble_connection_find_conference_server (priv->conn);
+  g_assert (priv->roomlist_channel == NULL);
 
-      if (server == NULL)
-        return FALSE;
+  server = _gabble_connection_find_conference_server (priv->conn);
 
-      object_path = g_strdup_printf ("%s/RoomlistChannel", priv->conn->object_path);
+  if (server == NULL)
+    return FALSE;
 
-      priv->roomlist_channel = _gabble_roomlist_channel_new (priv->conn,
-          object_path, server);
+  object_path = g_strdup_printf ("%s/RoomlistChannel",
+      conn->object_path);
 
-      g_signal_connect (priv->roomlist_channel, "closed",
-                        (GCallback) roomlist_channel_closed_cb, fac);
+  priv->roomlist_channel = _gabble_roomlist_channel_new (priv->conn,
+      object_path, server);
 
-      g_signal_emit_by_name (fac, "new-channel", priv->roomlist_channel);
+  g_signal_connect (priv->roomlist_channel, "closed",
+                    (GCallback) roomlist_channel_closed_cb, fac);
 
-      g_free (object_path);
-    }
+  tp_channel_factory_iface_emit_new_channel (fac,
+      (TpChannelIface *)priv->roomlist_channel, request);
+
+  g_free (object_path);
 
   return TRUE;
 }
@@ -711,15 +862,16 @@ gabble_muc_factory_iface_connecting (TpChannelFactoryIface *iface)
   g_assert (priv->message_cb == NULL);
   g_assert (priv->presence_cb == NULL);
 
-  priv->message_cb = lm_message_handler_new (muc_factory_message_cb, fac, NULL);
+  priv->message_cb = lm_message_handler_new (muc_factory_message_cb, fac,
+      NULL);
   lm_connection_register_message_handler (priv->conn->lmconn, priv->message_cb,
                                           LM_MESSAGE_TYPE_MESSAGE,
                                           LM_HANDLER_PRIORITY_NORMAL);
 
-  priv->presence_cb = lm_message_handler_new (muc_factory_presence_cb, fac, NULL);
-  lm_connection_register_message_handler (priv->conn->lmconn, priv->presence_cb,
-                                          LM_MESSAGE_TYPE_PRESENCE,
-                                          LM_HANDLER_PRIORITY_NORMAL);
+  priv->presence_cb = lm_message_handler_new (muc_factory_presence_cb,
+      fac, NULL);
+  lm_connection_register_message_handler (priv->conn->lmconn,
+      priv->presence_cb, LM_MESSAGE_TYPE_PRESENCE, LM_HANDLER_PRIORITY_NORMAL);
 }
 
 
@@ -740,13 +892,13 @@ gabble_muc_factory_iface_disconnected (TpChannelFactoryIface *iface)
   g_assert (priv->message_cb != NULL);
   g_assert (priv->presence_cb != NULL);
 
-  lm_connection_unregister_message_handler (priv->conn->lmconn, priv->message_cb,
-                                            LM_MESSAGE_TYPE_MESSAGE);
+  lm_connection_unregister_message_handler (priv->conn->lmconn,
+      priv->message_cb, LM_MESSAGE_TYPE_MESSAGE);
   lm_message_handler_unref (priv->message_cb);
   priv->message_cb = NULL;
 
-  lm_connection_unregister_message_handler (priv->conn->lmconn, priv->presence_cb,
-                                            LM_MESSAGE_TYPE_PRESENCE);
+  lm_connection_unregister_message_handler (priv->conn->lmconn,
+      priv->presence_cb, LM_MESSAGE_TYPE_PRESENCE);
   lm_message_handler_unref (priv->presence_cb);
   priv->presence_cb = NULL;
 }
@@ -767,7 +919,9 @@ _foreach_slave (gpointer key, gpointer value, gpointer user_data)
 }
 
 static void
-gabble_muc_factory_iface_foreach (TpChannelFactoryIface *iface, TpChannelFunc foreach, gpointer user_data)
+gabble_muc_factory_iface_foreach (TpChannelFactoryIface *iface,
+                                  TpChannelFunc foreach,
+                                  gpointer user_data)
 {
   GabbleMucFactory *fac = GABBLE_MUC_FACTORY (iface);
   GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (fac);
@@ -787,23 +941,32 @@ gabble_muc_factory_iface_request (TpChannelFactoryIface *iface,
                                   const gchar *chan_type,
                                   TpHandleType handle_type,
                                   guint handle,
+                                  gpointer request,
                                   TpChannelIface **ret,
                                   GError **error)
 {
   GabbleMucFactory *fac = GABBLE_MUC_FACTORY (iface);
   GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (fac);
+  TpHandleRepoIface *room_repo = tp_base_connection_get_handles (
+      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_ROOM);
   GabbleMucChannel *chan;
 
-  if (!g_strdiff (chan_type, TP_IFACE_CHANNEL_TYPE_ROOM_LIST))
+  if (!tp_strdiff (chan_type, TP_IFACE_CHANNEL_TYPE_ROOM_LIST))
     {
+      if (priv->roomlist_channel != NULL)
+        {
+          *ret = TP_CHANNEL_IFACE (priv->roomlist_channel);
+          return TP_CHANNEL_FACTORY_REQUEST_STATUS_EXISTING;
+        }
+
       /* FIXME - delay if services aren't discovered yet? */
-      if (!make_roomlist_channel (fac))
+      if (!make_roomlist_channel (fac, request))
         {
           DEBUG ("no conference server available for roomlist request");
           return TP_CHANNEL_FACTORY_REQUEST_STATUS_NOT_AVAILABLE;
         }
       *ret = TP_CHANNEL_IFACE (priv->roomlist_channel);
-      return TP_CHANNEL_FACTORY_REQUEST_STATUS_DONE;
+      return TP_CHANNEL_FACTORY_REQUEST_STATUS_CREATED;
     }
 
   if (strcmp (chan_type, TP_IFACE_CHANNEL_TYPE_TEXT))
@@ -812,7 +975,7 @@ gabble_muc_factory_iface_request (TpChannelFactoryIface *iface,
   if (handle_type != TP_HANDLE_TYPE_ROOM)
     return TP_CHANNEL_FACTORY_REQUEST_STATUS_NOT_AVAILABLE;
 
-  if (!gabble_handle_is_valid (priv->conn->handles, TP_HANDLE_TYPE_ROOM, handle, NULL))
+  if (!tp_handle_is_valid (room_repo, handle, NULL))
     return TP_CHANNEL_FACTORY_REQUEST_STATUS_INVALID_HANDLE;
 
   chan = g_hash_table_lookup (priv->channels, GINT_TO_POINTER (handle));
@@ -825,7 +988,7 @@ gabble_muc_factory_iface_request (TpChannelFactoryIface *iface,
   if (_gabble_muc_channel_is_ready (chan))
     {
       *ret = TP_CHANNEL_IFACE (chan);
-      return TP_CHANNEL_FACTORY_REQUEST_STATUS_DONE;
+      return TP_CHANNEL_FACTORY_REQUEST_STATUS_EXISTING;
     }
   else
     {
