@@ -1,18 +1,19 @@
 
 #include <string.h>
+#include <unistd.h>
 
 #include <glib.h>
-#include <gabble-presence.h>
+#include <presence.h>
 
-int main(int argc, char **argv)
+int main (int argc, char **argv)
 {
   const gchar *resource;
   gchar *dump;
   GabblePresence *presence;
 
-  g_type_init();
+  g_type_init ();
 
-  presence = gabble_presence_new();
+  presence = gabble_presence_new ();
   g_assert (GABBLE_PRESENCE_OFFLINE == presence->status);
   g_assert (NULL == presence->status_message);
 
@@ -42,6 +43,9 @@ int main(int argc, char **argv)
   g_assert (FALSE == gabble_presence_update (presence, "foo",
     GABBLE_PRESENCE_AVAILABLE, "status message", 0));
 
+  /* sleep a while so the next resource will have different timestamp */
+  sleep (1);
+
   /* presence from different resource, but equal present-ness; unchanged */
   g_assert (FALSE == gabble_presence_update (presence, "bar",
     GABBLE_PRESENCE_AVAILABLE, "dingoes", 0));
@@ -49,6 +53,13 @@ int main(int argc, char **argv)
   g_assert (GABBLE_PRESENCE_AVAILABLE == presence->status);
   g_assert (0 == strcmp ("status message", presence->status_message));
 
+  /* but if we were to make a voip call, we would prefer the newer one */
+  g_assert (0 == strcmp ("bar",
+        gabble_presence_pick_resource_by_caps (presence, 0)));
+
+  /* sleep a while so the next resource will have different timestamp */
+  sleep (1);
+
   /* presence with higher priority; presence and message changed */
   g_assert (TRUE == gabble_presence_update (presence, "bar",
     GABBLE_PRESENCE_AVAILABLE, "dingoes", 1));
@@ -56,10 +67,33 @@ int main(int argc, char **argv)
   g_assert (GABBLE_PRESENCE_AVAILABLE == presence->status);
   g_assert (0 == strcmp ("dingoes", presence->status_message));
 
+  /* sleep a while so the next resource will have different timestamp */
+  sleep (1);
+
+  /* now foo is newer, so the next voip call would prefer that */
+  g_assert (FALSE == gabble_presence_update (presence, "foo",
+    GABBLE_PRESENCE_AVAILABLE, "status message", 0));
+  g_assert (0 == strcmp ("foo",
+        gabble_presence_pick_resource_by_caps (presence, 0)));
+
+  /* sleep a while so the next resource will have different timestamp */
+  sleep (1);
+
   /* presence from first resource with greated present-ness: change */
   g_assert (TRUE == gabble_presence_update (presence, "foo",
     GABBLE_PRESENCE_CHAT, "status message", 0));
 
+  /* sleep a while so the next resource will have different timestamp */
+  sleep (1);
+
+  /* make bar be the latest presence: no change, since foo is more present */
+  g_assert (FALSE == gabble_presence_update (presence, "bar",
+    GABBLE_PRESENCE_AVAILABLE, "dingoes", 1));
+
+  /* we still prefer foo for the voip calls, because it's more present */
+  g_assert (0 == strcmp ("foo",
+        gabble_presence_pick_resource_by_caps (presence, 0)));
+
   g_assert (GABBLE_PRESENCE_CHAT == presence->status);
   g_assert (0 == strcmp ("status message", presence->status_message));
 
@@ -69,9 +103,10 @@ int main(int argc, char **argv)
   g_assert (NULL == resource);
 
   /* give voice cap to second resource, but make priority negative */
-  g_assert (FALSE == gabble_presence_update(presence, "bar",
+  g_assert (FALSE == gabble_presence_update (presence, "bar",
     GABBLE_PRESENCE_AVAILABLE, "dingoes", -1));
-  gabble_presence_set_capabilities (presence, "bar", PRESENCE_CAP_GOOGLE_VOICE, 0);
+  gabble_presence_set_capabilities (presence, "bar",
+      PRESENCE_CAP_GOOGLE_VOICE, 0);
 
   dump = gabble_presence_dump (presence);
   g_assert (0 == strcmp (dump,
@@ -99,7 +134,8 @@ int main(int argc, char **argv)
   g_assert (NULL == resource);
 
   /* give voice cap to first resource */
-  gabble_presence_set_capabilities (presence, "foo", PRESENCE_CAP_GOOGLE_VOICE, 0);
+  gabble_presence_set_capabilities (presence, "foo",
+      PRESENCE_CAP_GOOGLE_VOICE, 0);
 
   /* resource has voice cap */
   resource = gabble_presence_pick_resource_by_caps (presence,
@@ -111,7 +147,7 @@ int main(int argc, char **argv)
   g_assert (TRUE == gabble_presence_update (presence, NULL,
     GABBLE_PRESENCE_OFFLINE, "gone", 0));
   g_assert (GABBLE_PRESENCE_OFFLINE == presence->status);
-  g_assert (0 == strcmp("gone", presence->status_message));
+  g_assert (0 == strcmp ("gone", presence->status_message));
 
   /* caps are gone too */
   resource = gabble_presence_pick_resource_by_caps (presence,
@@ -129,7 +165,7 @@ int main(int argc, char **argv)
     "  (none)\n"));
   g_free (dump);
 
-
+  g_object_unref (presence);
   return 0;
 }
 
